package	jp.co.sds_corp.hibiya.connector;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

import jp.co.sds_corp.hibiya.server.IChecker;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.IService;
import jp.co.sds_corp.util.daemon.manager.DaemonManager;
import jp.co.sds_corp.util.daemon.remote.IRemoteClient;
import jp.co.sds_corp.util.daemon.remote.RemoteFactory;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* {@link DaemonManager} のI/Fを使用して{@link IChecker}の情報を取得します。
*/
public class DaemonConnector extends AbstractConnector
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	IRemoteClient	client;

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		client = RemoteFactory.createClient( config );
	}

	public void stop() {
		super.stop();
		logger.debug( " stopped.[" + getName() + "]" );
	}

	/**
	* 接続状態を検査します。
	*/
	public boolean checkConnection() throws Exception {
		client.execRemote( "status" );
		return true;
	}

	/**
	* {@link IChecker}を取得します。
	* @param	name	ICheckerの登録名
	*/
	public IChecker getChecker( String name ) {
		Map<String,IService> map = null;
		try {
			map = (Map<String,IService> )client.execRemote( "getServiceMap" );
		} catch ( Exception ex ) {
			logger.error( "getChecker() error ", ex );
			return null;
		}
		return (IChecker )map.get( name );
	}

	/**
	* {@link IChecker}の一覧を返します。
	* @return	ICheckerの一覧が格納されたMap
	*/
	public Map<String,IChecker> getCheckerMap() {
		LinkedHashMap<String,IChecker> ret = new LinkedHashMap<String,IChecker>();
		Map<String,IService> map = null;
		try {
			map = (Map<String,IService> )client.execRemote( "getServiceMap" );
		} catch ( Exception ex ) {
			logger.error( "getCheckerMap() error ", ex );
			return null;
		}
		Iterator<String> itr = map.keySet().iterator();
		while( itr.hasNext() ) {
			String key = (String )itr.next();
			IService svc = (IService )map.get( key );
			if ( svc instanceof IChecker ) {
				ret.put( svc.getName(), (IChecker )svc );
			}
		}
		return ret;
	}

	/**
	* {@link IChecker}を追加します
	* @param	chk		追加するIChecker
	*/
	public void addChecker( IChecker chk ) {
	}

	/**
	* {@link IChecker}を削除します。
	* @param	chk		削除するIChecker
	*/
	public void removeChecker( IChecker chk ) {
	}

}
