package	jp.co.sds_corp.hibiya.server;

import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Vector;

import jp.co.sds_corp.hibiya.action.IAction;
import jp.co.sds_corp.hibiya.filter.IFilter;
import jp.co.sds_corp.hibiya.threshold.IThreshold;
import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.AbstractService;
import jp.co.sds_corp.util.daemon.config.ConfigUtility;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* 監視サービスの基底クラスです。
*/
public abstract class AbstractChecker extends AbstractService
	implements IChecker
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	IChecker.Status	checkerStatus = IChecker.Status.INIT;
	String	type;
	long	interval;
	Map<Object,Object>	mapRes =
		Collections.synchronizedMap( new LinkedHashMap<Object,Object>());
	Vector<IAction> vecAct = new Vector<IAction>();
	IThreshold	threshold;
	transient Vector<ICheckerListener> vecStat = new Vector<ICheckerListener>();
	boolean	debug;	// デバッグフラグ
	boolean	saveHistory;	// 履歴保存フラグ
	List<CheckerChangeEvent> history = new Vector<CheckerChangeEvent>();	// 履歴
	transient ResourceBundle res = ResourceBundle.getBundle( "resource.server" );
	int		hash = hashCode();

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		debug = config.getAttributes().getBoolean( "debug", false );
		saveHistory = config.getAttributes().getBoolean( "history", true );
		Iterator<IConfiguration> itr;
		itr = config.getChilds( "Action" );
		while ( itr.hasNext() ) {
			IConfiguration c = (IConfiguration )itr.next();
			IAction act = createAction( c );
			addAction( act );
		}
		itr = config.getChilds( "Threshold" );
		while ( itr.hasNext() ) {
			IConfiguration c = (IConfiguration )itr.next();
			IThreshold thr = createThreshold( c );
			setThreshold( thr );
		}
	}

	/**
	* 監視処理を開始します。
	*/
	public void start() {
		try {
			check();

			String	text;
			String msg = null;
			IChecker.Status nPrev = getPrevStatus();
			if ( getCheckerStatus() == IChecker.Status.INIT ) {
				text = res.getString( "checker.init" );
				msg = getType() + " [" + getName() + "] " + text;
			} else {
				String thrMsg = null;
				if ( getThreshold() != null ) {
					IChecker.Status thrstat = getThreshold().getCheckerStatus( this );
					if ( thrstat.compareTo( getCheckerStatus() ) > 0 ) {
						setCheckerStatus( thrstat );
						thrMsg = getThreshold().getMessage();
					}
				}
				if ( nPrev != getCheckerStatus() || nPrev == IChecker.Status.INIT ) {
					if ( getCheckerStatus() == IChecker.Status.OK ) {
						text = res.getString( "checker.recover" );
						msg = getType() + " [" + getName()
								+ "] " + text;
						if ( thrMsg != null ) {
							msg += " (" + thrMsg + ") ";
						}
					} else if ( getCheckerStatus() == IChecker.Status.WARN ) {
						text = res.getString( "checker.warn" );
						msg = getType() + " [" + getName()
								+ "] " + text;
						if ( thrMsg != null ) {
							msg += " (" + thrMsg + ") ";
						}
						if ( getException() != null ) {
							msg += "(" + getException() + ")";
						}
					} else {
						text = res.getString( "checker.error" );
						msg = getType() + " [" + getName()
								+ "] " + text;
						if ( thrMsg != null ) {
							msg += " (" + thrMsg + ") ";
						}
						if ( getException() != null ) {
							msg += "(" + getException() + ")";
						}
					}
				}
			}
			setPrevStatus( getCheckerStatus() );
			if ( msg == null ) {
				msg = "not changed.";
			}
			if ( msg != null ) {
				fireCheckerChange( this, nPrev, getCheckerStatus(), getException(), msg );
			}
		} catch ( Exception e ) {
			logger.error( "monitor error", e );
			setCheckerStatus( IChecker.Status.NG );
		}
	}

	/**
	* 状態を設定します。
	* @param	var		設定する状態
	*/
	public void setCheckerStatus( IChecker.Status var ) {
		this.checkerStatus = var;
	}
	public IChecker.Status getCheckerStatus() {
		return checkerStatus;
	}
	/**
	* 状態を初期化します。
	*/
	public void clearCheckerStatus() {
		setCheckerStatus( IChecker.Status.INIT );
		setPrevStatus( IChecker.Status.INIT );
	}

	IChecker.Status nPrev = IChecker.Status.INIT;
	/**
	* 直前の状態を設定します。
	* @param	var		直前の状態
	*/
	protected void setPrevStatus( IChecker.Status var ) {
		this.nPrev = var;
	}
	/**
	* 直前の状態を返します。
	*/
	protected IChecker.Status getPrevStatus() {
		return this.nPrev;
	}

	/**
	* 監視処理を停止します。
	*/
	public void stop() {
		try {
			String txt = res.getString( "checker.stop" );
			String msg = getType() + " [" + getName() + "] " + txt;
//			fireCheckerChange( this, getCheckerStatus(), IChecker.Status.NG, null, msg );
		} catch ( Exception ex ) {
			logger.error( "stop error", ex );
		}
		logger.info( "stopped." );
	}

	/**
	* 設定されているアクションを生成します。
	* @param	config	設定情報
	*/
	public IAction createAction( IConfiguration config ) throws Exception {
		IAction act = (IAction )ConfigUtility.createInstance( config );
		act.init( config );
		return act;
	}

	/**
	* 設定されているIThresholdを生成します。
	* @param	config	設定情報
	*/
	public IThreshold createThreshold( IConfiguration config ) throws Exception{
		IThreshold thr = (IThreshold )ConfigUtility.createInstance( config );
		thr.init( config );
		return thr;
	}

	/**
	* 監視タイプ(type)を設定します。
	* @param	var		設定するデータ
	*/
	public void setType( String var ) {
		this.type = var ;
	}
	/**
	* 監視タイプ(type)を取得します。
	* @return	typeの内容
	*/
	public String getType() {
		return type;
	}

	/**
	* 例外(exception)を設定します。
	* @param	var		exceptionの内容
	*/
	public void setException( Throwable var ) {
		putResult( "Exception", var );
	}
	/**
	* 例外(exception)を取得します。
	* @return	exceptionの内容
	*/
	public Throwable getException() {
		Throwable ex = (Throwable )getResult( "Exception" );
		return ex;
	}
	/**
	* 処理時間を設定します。
	* @param	start	開始時刻
	* @param	end		終了時刻
	*/
	public void setResponseTime( CmnDate start, CmnDate end ) {
		setResponseTime( end.getTime().getTime() - start.getTime().getTime() );
	}
	/**
	* 処理結果を設定します。
	* @param	var		処理結果
	*/
	public void setResponseTime( long var ) {
		mapRes.put( "ResponseTime", String.valueOf( var ));
	}
	/**
	* 処理結果を取得します。
	* @return	処理結果
	*/
	public synchronized long getResponseTime() {
		String resp = (String )mapRes.get( "ResponseTime" );
		if ( resp == null ) {
			return 0;
		}
		return Long.valueOf( resp ).longValue();
	}

	/**
	* チェック結果を取得します。
	* @param	key		キー
	* @return	チェック結果
	*/
	public synchronized Object getResult( Object key ) {
		return mapRes.get( key );
	}

	/**
	* チェック結果を設定します。
	* @param	key		キー
	* @param	value	チェック結果
	*/
	public void putResult( Object key, Object value ) {
		mapRes.put( (String)key, value );
	}

	/**
	* チェック結果を取得します。
	* @return	チェック結果
	*/
	public synchronized Map<Object,Object> getResults() {
		return mapRes;
	}

	/**
	* チェック結果を消去します。
	*/
	public void clearResult() {
		mapRes.clear();
		setException( null );
	}

	/**
	* 監視間隔(interval)を設定します。
	* @param	var		設定するデータ
	*/
	public void setInterval( long var ) {
		this.interval = var ;
	}
	/**
	* 監視間隔(interval)を取得します。
	* @return	intervalの内容
	*/
	public long getInterval() {
		return interval;
	}

	/**
	* 状態の変更を受け取るオブジェクトを追加します。
	* @param	l		状態の変更を受け取るオブジェクト
	*/
	public void addCheckerListener( ICheckerListener l ) {
		if ( vecStat != null ) {
			vecStat.add( l );
		}
	}

	/**
	* 状態の変更を受け取るオブジェクトを削除します。
	* @param	l		状態の変更を受け取るオブジェクト
	*/
	public void removeCheckerListener( ICheckerListener l ) {
		if ( vecStat != null ) {
			vecStat.remove( l );
		}
	}

	/**
	* 状態変更イベントを通知します。
	* @param	source		イベントの発生元
	* @param	nOldStat	変更前の状態
	* @param	nNewStat	変更後の状態
	* @param	exc			発生した例外
	* @param	msg			メッセージ
	*/
	public void fireCheckerChange( Object source,
			IChecker.Status nOldStat, IChecker.Status nNewStat,
			Throwable exc, String msg )
		throws Exception
	{
		CheckerChangeEvent ev = new CheckerChangeEvent(
								source, nOldStat, nNewStat, exc, msg );
		fireCheckerChange( ev );
	}

	/**
	* 状態変更イベントを通知します。
	* @param	ev		状態変更イベント
	*/
	public void fireCheckerChange( CheckerChangeEvent ev ) throws Exception {
		checkerChanged( ev );
	}

	public void checkerChanged( CheckerChangeEvent ev ) {
		String msg = "checker changed. " + ev.getMessage();
		if ( ev.getOldStatus() == ev.getNewStatus()) {
			msg = ev.getMessage();
		}
		if ( ev.getException() != null ) {
			msg += "[" + ev.getException() + "]";
		}
		if ( isDebug()) {
			logger.debug( msg );
		}
		if ( ev.getNewStatus() != ev.getOldStatus() ) {
			if ( isSaveHistory()) {
				history.add( ev );
			}
		}

		Iterator<IAction> itr = vecAct.iterator();
		while ( itr.hasNext() ) {
			IAction act = (IAction )itr.next();
			if ( !act.isValid()) {
				continue;
			}

			IFilter flt = act.getFilter();
			if ( flt == null || flt.match( ev )) {
				act.action( ev );
			}
		}
		for ( int i = 0; i < vecStat.size(); i++ ) {
			ICheckerListener l = (ICheckerListener )vecStat.get( i );
			try {
				l.checkerChanged( ev );
			} catch ( Exception ex ) {
				logger.error( "error", ex );
			}
		}
	}

	/**
	* このサービスに設定されているアクションを全てクリアします。
	*/
	public void clearAction() {
		vecAct.clear();
	}

	/**
	* このサービスにアクションを追加します。
	* @param	var		追加するアクション
	*/
	public void addAction( IAction var ) {
		vecAct.add( var );
	}

	/**
	* このサービスに設定されているアクションを返します。
	* @return	このサービスに設定されているアクション
	*/
	public List<IAction> getActionList() {
		return vecAct;
	}

	/**
	* このサービスにアクションを追加します。
	* @param	var		追加するアクション
	*/
	public void setThreshold( IThreshold var ) {
		this.threshold = var;
	}

	/**
	* このサービスに設定されているアクションを返します。
	* @return	このサービスに設定されているアクション
	*/
	public IThreshold getThreshold() {
		return this.threshold;
	}

	/**
	* 履歴保存フラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setSaveHistory( boolean var ) {
		this.saveHistory = var;
	}
	/**
	* 履歴保存フラグを取得します。
	* @return	履歴保存フラグ
	*/
	public boolean isSaveHistory() {
		return saveHistory;
	}

	/**
	* デバッグフラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setDebug( boolean var ) {
		this.debug = var;
	}
	/**
	* デバッグフラグを取得します。
	* @return	履歴保存フラグ
	*/
	public boolean isDebug() {
		return debug;
	}

	/**
	* 履歴を取得します。
	* @return	履歴
	*/
	public List<CheckerChangeEvent> listHistory() {
		return history;
	}

	/**
	* hash値を取得します。
	* @return	hash値
	*/
	public int getHashCode() {
		return hash;
	}


	/**
	* 設定内容を保存します。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		if ( !isSaveHistory()) {
			config.setAttribute( "history", "false" );
		}
		IConfiguration c;
		for ( int i = 0; i < vecAct.size(); i++ ) {
			IAction act = (IAction )vecAct.get( i );
			c = config.addChild( "Action" );
			act.saveConfig( c );
		}
		if ( threshold != null ) {
			c = config.addChild( "Threshold" );
			threshold.saveConfig( c );
		}
	}

	/**
	* このモニタの文字列表現を返します。
	* @return	このモニタの文字列表現
	*/
	public String toString() {
		String ret = super.toString();
		ret += ":int=" + interval;
		return ret;
	}

}
