package	jp.co.sds_corp.hibiya.action;

import java.util.Iterator;
import java.util.Vector;

import jp.co.sds_corp.hibiya.server.CheckerChangeEvent;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.mail.MailSender;

/**
メールを送信するアクションです。
XMLは以下の形式です。
<TABLE BORDER="1">
	<TR><TH>要素</TH><TH>必須</TH><TH>説明</TH><TH>備考</TH></TR>
	<TR><TD>Server</TD><TD>※</TD><TD>SMTPサーバ</TD><TD>&nbsp;</TD></TR>
	<TR><TD>From</TD><TD>※</TD><TD>Mailの送信者</TD><TD>&nbsp;</TD></TR>
	<TR><TD>To</TD><TD>&nbsp;</TD><TD>Mailの宛先(TO)</TD><TD>&nbsp;</TD></TR>
	<TR><TD>Bcc</TD><TD>&nbsp;</TD><TD>Mailの宛先(BCC)</TD><TD>&nbsp;</TD></TR>
	<TR><TD>Subject</TD><TD>&nbsp;</TD><TD>Mailのタイトル</TD><TD>&nbsp;</TD></TR>
	<TR>
		<TD>Layout</TD><TD>&nbsp;</TD>
		<TD>formatに使用するLayout</TD>
		<TD>指定されない場合はメッセージのみ出力</TD>
	</TR>
</TABLE>
<PRE>
&lt;Action ss="jp.co.sds_corp.hibiya.action.MailAction"&gt;
	&lt;Server&gt;smtp.foo.co.jp&lt;/Server&gt;
	&lt;From&gt;from@bar.co.jp&lt;/From&gt;
	&lt;To&gt;to1@bar.co.jp&lt;/To&gt;
	&lt;To&gt;to2@bar.co.jp&lt;/To&gt;
	&lt;Cc&gt;bcc1@bar.co.jp&lt;/Cc&gt;
	&lt;Bcc&gt;bcc1@bar.co.jp&lt;/Bcc&gt;
	&lt;Bcc&gt;bcc2@bar.co.jp&lt;/Bcc&gt;
	&lt;Subject&gt;Hibiya report.&lt;/Subject&gt;
	&lt;Layout class="jp.co.sds_corp.hibiya.layout.EventLayout"&gt;
		%h からsyslogを受信しました。
		%f:%p [%m]
	&lt;/Layout&gt;
&lt;/Action&gt;
</PRE>
*/
public class MailAction extends AbstractAction
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	String	server;
	String	from;
	String	subject;
	Vector<String>	vecTo = new Vector<String>();
	Vector<String>	vecCc = new Vector<String>();
	Vector<String>	vecBcc = new Vector<String>();

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		setServer( config.get( "Server" ));
		setFrom( config.get( "From" ));
		Iterator<IConfiguration> itr;
		itr = config.getChilds( "To" );
		while ( itr.hasNext() ) {
			IConfiguration c = (IConfiguration )itr.next();
			addTo( c.get() );
		}
		itr = config.getChilds( "Cc" );
		while ( itr.hasNext() ) {
			IConfiguration c = (IConfiguration )itr.next();
			addCc( c.get() );
		}
		itr = config.getChilds( "Bcc" );
		while ( itr.hasNext() ) {
			IConfiguration c = (IConfiguration )itr.next();
			addBcc( c.get() );
		}
		setSubject( config.get( "Subject", null ));
	}

	/**
	* アクションを実行します。
	*/
	public void action( CheckerChangeEvent ev ) {
		String subject = getSubject();
		if ( subject == null ) {
			subject = "Hibiya report";
		}
		String s;
		if ( getLayout() == null ) {
			s = ev.getMessage();
		} else {
			s = getLayout().format( ev );
		}

		MailSender ms = new MailSender();
		ms.setHost( getServer() );
		ms.setFrom( getFrom() );
		for ( int i = 0; i < vecTo.size(); i++ ) {
			ms.addTo( (String )vecTo.get( i ));
		}
		for ( int i = 0; i < vecCc.size(); i++ ) {
			ms.addCc( (String )vecCc.get( i ));
		}
		for ( int i = 0; i < vecBcc.size(); i++ ) {
			ms.addBcc( (String )vecBcc.get( i ));
		}
		ms.setSubject( subject );
		ms.setMessage( s );
		try {
			ms.send();
		} catch ( Exception ex ) {
			logger.error( "error", ex );
		}
	}

	/**
	* 設定内容の文字列表現を返します。
	*/
	public String paramString() {
		String ret = super.paramString();
		ret += server + ":" + from + ":" + ":" + subject;
		for ( int i = 0; i < vecTo.size(); i++ ) {
			ret += ":T=" + vecTo.get( i );
		}
		for ( int i = 0; i < vecCc.size(); i++ ) {
			ret += ":C=" + vecCc.get( i );
		}
		for ( int i = 0; i < vecBcc.size(); i++ ) {
			ret += ":B=" + vecBcc.get( i );
		}
		return ret;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	親ノード
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		IConfiguration c;
		c = config.addChild( "Server", getServer() );
		c = config.addChild( "From", getFrom() );
		Vector<String> vec;
		vec = getTo();
		for ( int i = 0; i < vec.size(); i++ ) {
			c = config.addChild( "To", (String )vec.get( i ) );
		}
		vec = getCc();
		for ( int i = 0; i < vec.size(); i++ ) {
			c = config.addChild( "Cc", (String )vec.get( i ) );
		}
		vec = getBcc();
		for ( int i = 0; i < vec.size(); i++ ) {
			c = config.addChild( "Bcc", (String )vec.get( i ) );
		}
		if ( getSubject() != null ) {
			c = config.addChild( "Subject", getSubject() );
		}
	}

	/**
	* SMTPサーバ名(server)を設定します。
	* @param	var		設定するデータ
	*/
	public void setServer( String var ) {
		this.server = var ;
	}
	/**
	* SMTPサーバ名(server)を取得します。
	* @return	serverの内容
	*/
	public String getServer() {
		return server;
	}

	/**
	* 送信元アドレス(from)を設定します。
	* @param	var		設定するデータ
	*/
	public void setFrom( String var ) {
		this.from = var ;
	}
	/**
	* 送信元アドレス(from)を取得します。
	* @return	fromの内容
	*/
	public String getFrom() {
		return from;
	}

	/**
	* 送信先アドレス(to)を追加します。
	* @param	var		設定するデータ
	*/
	public void addTo( String var ) {
		vecTo.add( var );
	}
	/**
	* 送信先アドレス(to)を取得します。
	* @return	toの内容
	*/
	public Vector<String> getTo() {
		return vecTo;
	}
	/**
	* 送信先アドレス(to)をクリアします。
	*/
	public void clearTo() {
		vecTo.clear();
	}

	/**
	* 送信先アドレス(cc)を追加します。
	* @param	var		設定するデータ
	*/
	public void addCc( String var ) {
		vecCc.add( var );
	}
	/**
	* 送信先アドレス(cc)を取得します。
	* @return	bccの内容
	*/
	public Vector<String> getCc() {
		return vecCc;
	}
	/**
	* 送信先アドレス(cc)をクリアします。
	*/
	public void clearCc() {
		vecCc.clear();
	}

	/**
	* 送信先アドレス(bcc)を追加します。
	* @param	var		設定するデータ
	*/
	public void addBcc( String var ) {
		vecBcc.add( var );
	}
	/**
	* 送信先アドレス(bcc)を取得します。
	* @return	bccの内容
	*/
	public Vector<String> getBcc() {
		return vecBcc;
	}
	/**
	* 送信先アドレス(bcc)をクリアします。
	*/
	public void clearBcc() {
		vecBcc.clear();
	}

	/**
	* タイトル(subject)を設定します。
	* @param	var		設定するデータ
	*/
	public void setSubject( String var ) {
		this.subject = var ;
	}
	/**
	* タイトル(subject)を取得します。
	* @return	subjectの内容
	*/
	public String getSubject() {
		return subject;
	}

}
